@@ -59,6 +59,8 @@ trend_line <- function(
   ){
 
   `%>%` <- magrittr::`%>%`
+  stop_if_no_config()
+  stop_if_no_fs()
   # If countries are names they have to be changes to region indexes
   if(is.character(countries) && length(countries) > 0) {
     reg <- get_country_items()
@@ -87,7 +89,8 @@ trend_line <- function(
     )
   # checking if some data points have been returned or return empty char
   if(nrow(df %>% dplyr::filter(.data$number_of_tweets > 0)) >0) {
-    df$topic <- unname(get_topics_labels()[stringr::str_replace_all(topic, "%20", " ")])
+    topic <- unname(get_topics_labels()[stringr::str_replace_all(topic, "%20", " ")])
+    df$topic <- topic 
     plot_trendline(
       df = df,
       countries = countries,
@@ -212,7 +215,8 @@ plot_trendline <- function(df,countries,topic,date_min,date_max, date_type, alph
           # Week format if period is between 16 days and 20 weeks
 			    #  - Case for period ending on same day of week than period start 
 			    } else if(weeks <= 20) {
-            "%G-w%V"
+            "%Y-%m-%d"
+            #"%G-w%V"
           # Month format day of month in period start if period is less or equal to 2 years but more than 20 weeks
           } else if(years <=2) {
             "%Y-%b"
@@ -327,6 +331,8 @@ plot_trendline <- function(df,countries,topic,date_min,date_max, date_type, alph
 #' @importFrom ggplot2 fortify theme element_text element_blank element_rect ggplot geom_polygon aes geom_point scale_size_continuous scale_fill_manual coord_fixed labs theme_classic
 #' @importFrom stats setNames 
 create_map <- function(topic=c(),countries=c(1), date_min="1900-01-01",date_max="2100-01-01", with_retweets = FALSE, location_type = "tweet", caption = "", proj = NULL, forplotly=FALSE){
+  stop_if_no_config()
+  stop_if_no_fs()
   # Importing pipe operator
   `%>%` <- magrittr::`%>%`
   # Setting the scientific pen off
@@ -334,10 +340,10 @@ create_map <- function(topic=c(),countries=c(1), date_min="1900-01-01",date_max=
   on.exit(options(old))
   options(scipen=999)
 
-  # getting all regiond
+  # getting all regions
   regions <- get_country_items()
   
-  # If countries are names they have to be changes to region indexes
+  # If countries are names, they have to be changed to region indexes
   if(is.character(countries) && length(countries) > 0) {
     countries = (1:length(regions))[sapply(1:length(regions), function(i) regions[[i]]$name %in% countries)]
   }
@@ -359,6 +365,7 @@ create_map <- function(topic=c(),countries=c(1), date_min="1900-01-01",date_max=
     return(get_empty_chart("No data found for the selected topic, region and period"))
   }
   
+  
   #filtering data by topic and date and country_codes
   f_topic <- topic
   df <- (df %>% 
@@ -498,6 +505,9 @@ create_map <- function(topic=c(),countries=c(1), date_min="1900-01-01",date_max=
     return(get_empty_chart("No data found for the selected topic, region and period"))
   }
 
+  topic <- unname(get_topics_labels()[stringr::str_replace_all(topic, "%20", " ")])
+  df$topic <- topic 
+  
   # Adding country properties (bounding boxes and country names)
   regions <- get_country_items()
   map <- get_country_index_map()
@@ -639,8 +649,8 @@ create_map <- function(topic=c(),countries=c(1), date_min="1900-01-01",date_max=
     axis.ticks = ggplot2::element_blank(),
     axis.title.x = ggplot2::element_blank(),
     axis.title.y = ggplot2::element_blank(),
-    legend.direction="horizontal",
-    legend.position = "bottom"
+    legend.direction="vertical",
+    legend.position = "right"
   ))
 
   # creating the plot
@@ -712,7 +722,7 @@ create_map <- function(topic=c(),countries=c(1), date_min="1900-01-01",date_max=
 #' 
 #' This report may be empty for combinations of countries and topics with very few tweets since for performance reasons, the calculation of top words is an approximation using chunks of 10.000 tweets.
 #'
-#' This functions requires that \code{\link{search_loop}} and \code{\link{detect_loop}} have already been run successfully to show results.
+#' This function requires that \code{\link{search_loop}} and \code{\link{detect_loop}} have already been run successfully to show results.
 #' @examples 
 #' if(FALSE){
 #'    message('Please choose the epitweetr data directory')
@@ -755,8 +765,6 @@ create_topwords <- function(topic,country_codes=c(),date_min="1900-01-01",date_m
 #' @details Produces a bar chart showing the occurrences of the most popular words in the collected tweets based on the provided parameters.
 #' For performance reasons on tweet aggregation, this report only shows tweet location and ignores the location_type parameter
 #' 
-#' This report may be empty for combinations of countries and topics with very few tweets since for performance reasons, the calculation of top words is an approximation using chunks of 10.000 tweets.
-#'
 #' This functions requires that \code{\link{search_loop}} and \code{\link{detect_loop}} have already been run successfully to show results.
 #' @examples 
 #' if(FALSE){
@@ -784,6 +792,8 @@ create_topwords <- function(topic,country_codes=c(),date_min="1900-01-01",date_m
 #' @importFrom utils head
 #' 
 create_topchart <- function(topic, serie, country_codes=c(),date_min="1900-01-01",date_max="2100-01-01", with_retweets = FALSE, location_type = "tweet", top = 25) {
+  stop_if_no_config()
+  stop_if_no_fs()
   #Importing pipe operator
   `%>%` <- magrittr::`%>%`
   f_topic <- topic
@@ -844,6 +854,11 @@ create_topchart <- function(topic, serie, country_codes=c(),date_min="1900-01-01
   if(nrow(df)==0) {
     return(get_empty_chart("No data found for the selected topic, region and period"))
   }
+
+
+  topic <- unname(get_topics_labels()[stringr::str_replace_all(topic, "%20", " ")])
+  df$topic <- topic 
+  
   # Calculating breaks for y axis
   y_breaks <- unique(floor(pretty(seq(0, (max(df$frequency) + 1) * 1.1))))
   
