@@ -534,6 +534,53 @@ object schemas {
         , StructField("user_location_loc", geoLocationSchema,true)
         ))
     }
+    val geoLocatedTweetSchema = {
+      StructType(Seq(
+        StructField("topic",StringType,true)
+        , StructField("id",LongType,true)
+        , StructField("created_date",StringType,true)
+        , StructField("text",StringType,true)
+        , StructField("linked_text",StringType,true)
+        , StructField("user_description",StringType,true)
+        , StructField("linked_user_description",StringType,true)
+        , StructField("is_geo_located",BooleanType,true)
+        , StructField("is_retweet",BooleanType,true)
+        , StructField("lang",StringType,true)
+        , StructField("linked_lang",StringType,true)
+        , StructField("screen_name",StringType,true)
+        , StructField("user_name",StringType,true)
+        , StructField("user_id",LongType,true)
+        , StructField("user_location",StringType,true)
+        , StructField("linked_screen_name",StringType,true)
+        , StructField("linked_user_name",StringType,true)
+        , StructField("linked_user_location",StringType,true)
+        , StructField("created_at",TimestampType,true)
+        , StructField("tweet_longitude",DoubleType,true)  
+        , StructField("tweet_latitude",DoubleType,true)  
+        , StructField("linked_longitude",DoubleType,true)
+        , StructField("linked_latitude" ,DoubleType,true)
+        , StructField("place_type",StringType,true) 
+        , StructField("place_name",StringType,true) 
+        , StructField("place_full_name" ,StringType,true)
+        , StructField("linked_place_full_name" ,StringType,true)
+        , StructField("place_country_code"  ,StringType,true)   
+        , StructField("place_country"  ,StringType,true)        
+        , StructField("place_longitude",DoubleType,true)       
+        , StructField("place_latitude",DoubleType,true)        
+        , StructField("linked_place_longitude",DoubleType,true)
+        , StructField("linked_place_latitude",DoubleType,true) 
+        , StructField("linked_text_loc", geoLocationSchema,true)
+        , StructField("linked_place_full_name_loc", geoLocationSchema,true)
+        , StructField("place_full_name_loc", geoLocationSchema,true)
+        , StructField("text_loc", geoLocationSchema,true)
+        , StructField("user_description_loc", geoLocationSchema,true)
+        , StructField("user_location_loc", geoLocationSchema,true)
+        , StructField("hashtags", ArrayType(StringType, true),true)
+        , StructField("urls", ArrayType(StringType, true),true)
+        , StructField("entities", ArrayType(StringType, true),true)
+        , StructField("contexts", ArrayType(StringType, true),true)
+        ))
+    }
     val toIgnoreSchema = {
       StructType(Seq(
         StructField("file",StringType,true)
